"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const encode_object_1 = require("@batterii/encode-object");
const lodash_1 = require("lodash");
const invalid_cursor_error_1 = require("./invalid-cursor-error");
const nani_1 = require("nani");
class Cursor {
    constructor(query, sort, values) {
        this.query = query;
        this.sort = sort;
        this.values = values;
    }
    static fromObject(obj) {
        return new Cursor(obj.q, obj.s, obj.v);
    }
    static validateObject(value) {
        if (!lodash_1.isObjectLike(value)) {
            throw new invalid_cursor_error_1.InvalidCursorError('Cursor is not object-like', { info: { cursor: value } });
        }
        if (!lodash_1.isString(value.q)) {
            throw new invalid_cursor_error_1.InvalidCursorError('Cursor \'q\' is not a string', { info: { q: value.q } });
        }
        if (!lodash_1.isString(value.s)) {
            throw new invalid_cursor_error_1.InvalidCursorError('Cursor \'s\' is not a string', { info: { s: value.s } });
        }
        if (value.v !== undefined && !lodash_1.isArray(value.v)) {
            throw new invalid_cursor_error_1.InvalidCursorError('Cursor \'v\' is not an array', { info: { v: value.v } });
        }
        return value;
    }
    static parse(str) {
        let obj;
        try {
            obj = encode_object_1.decodeObject(str);
        }
        catch (err) {
            if (!nani_1.is(err, encode_object_1.InvalidJsonError))
                throw err;
            throw new invalid_cursor_error_1.InvalidCursorError({
                shortMessage: 'Cursor contains invalid JSON',
                cause: err,
                info: { cursor: str },
            });
        }
        return Cursor.fromObject(Cursor.validateObject(obj));
    }
    toObject() {
        const obj = { q: this.query, s: this.sort };
        if (this.values)
            obj.v = this.values;
        return obj;
    }
    serialize() {
        return encode_object_1.encodeObject(this.toObject());
    }
}
exports.Cursor = Cursor;
//# sourceMappingURL=cursor.js.map